﻿using DocumentFormat.OpenXml.Office2010.Excel;
using MotorPlantContracts.BindingModels;
using MotorPlantContracts.BusinessLogicsContracts;
using MotorPlantContracts.SearchModels;
using MotorPlantContracts.ViewModels;
using Microsoft.AspNetCore.Mvc;

namespace MotorPlantRestApi.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class MainController : Controller
    {
        private readonly ILogger _logger;
        private readonly IOrderLogic _order;
        private readonly IEngineLogic _Engine;
        public MainController(ILogger<MainController> logger, IOrderLogic order,
       IEngineLogic Engine)
        {
            _logger = logger;
            _order = order;
            _Engine = Engine;
        }
        [HttpGet]
        public List<EngineViewModel>? GetEngineList()
        {
            try
            {
                return _Engine.ReadList(null);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения списка мороженного");
                throw;
            }
        }
        [HttpGet]
        public EngineViewModel? GetEngine(int EngineId)
        {
            try
            {
                return _Engine.ReadElement(new EngineSearchModel
                {
                    Id =
			   EngineId
				});
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения продукта по id={Id}",
			   EngineId);
                throw;
            }
        }
        [HttpGet]
        public List<OrderViewModel>? GetOrders(int clientId)
        {
            try
            {
                return _order.ReadList(new OrderSearchModel
                {
                    ClientId = clientId
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения списка заказов клиента id ={ Id}", clientId);
            throw;
            }
        }
        [HttpPost]
        public void CreateOrder(OrderBindingModel model)
        {
            try
            {
                _order.CreateOrder(model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания заказа");
                throw;
            }
        }
    }
}
